Scorecard = Rock:NewAddon("Scorecard", "LibRockDB-1.0", "LibRockEvent-1.0", "LibRockConsole-1.0", "LibRockConfig-1.0", "LibFuBarPlugin-3.0")
local self = Scorecard
function Scorecard:OnEnable()
    Scorecard:AddEventListener("COMBAT_LOG_EVENT_UNFILTERED")
	Scorecard:AddSlashCommand(function() self:ToggleFrame() end, "/scorecard", "/score", "/sc")
	Scorecard:SetConfigTable(self.opts)
	self:RefreshConfig()
end

Scorecard:SetDatabase("ScorecardDB", "ScorecardDBC")
Scorecard:SetDatabaseDefaults('char', {
	records = {},
	options = {
		["lowerRank"] = true,
		["mySpells"] = true,
		["notMySpells"] = true,
		["splash"] = {
			["top"] = UIParent:GetTop()/2,
			["left"] = UIParent:GetRight()/2,
		},
		["output"] = "splash",
		["input"] = {
			["player"] = true,
			["pet"] = true,
			["guardian"] = true,
			["object"] = true,
		},
		["blacklist"] = {},
	}
})
function Scorecard:OnDisable()
	Scorecard:RemoveEventListener("COMBAT_LOG_EVENT_UNFILTERED")
	Scorecard:RemoveAllSlashCommands()
end

function Scorecard:OnInitialize() 
	self.SCROLLOFFSET = 0
	self.currentInfoSpell = 0
	self.timestampFormat = "%b %d, %y"
	self.maxHit = {
		["PvP"] =		 {["spellID"] = 0, ["dmg"] = 0},
		["PvE"] =		 {["spellID"] = 0, ["dmg"] = 0},
		["Vulnerable"] = {["spellID"] = 0, ["dmg"] = 0},
		["All"] =		 {["spellID"] = 0, ["dmg"] = 0},
	}
	self.maxCrit = {
		["PvP"] =		 {["spellID"] = 0, ["dmg"] = 0},
		["PvE"] =		 {["spellID"] = 0, ["dmg"] = 0},
		["Vulnerable"] = {["spellID"] = 0, ["dmg"] = 0},
		["All"] =		 {["spellID"] = 0, ["dmg"] = 0},
	}
	self.tabtext = {
	[1] = "PvE",
	[2] = "PvP",
	[3] = "Vulnerable",
	}
	self.colors = {
		[SCHOOL_MASK_NONE]		= {a=1,r=1,g=1,b=1};
		[SCHOOL_MASK_PHYSICAL]	= {r=(237/255),g=(165/255),b=(95/255)};
		[SCHOOL_MASK_HOLY] 		= {a=1,r=1,g=1,b=0.5};
		[SCHOOL_MASK_FIRE] 		= {a=1,r=1,g=0,b=60/255};
		[SCHOOL_MASK_NATURE] 	= {a=1,r=0,g=1,b=17/255};
		[SCHOOL_MASK_FROST] 	= {a=1,r=95/255,g=201/255,b=1};
		[SCHOOL_MASK_SHADOW] 	= {a=1,r=0.5,g=0,b=0.5};
		[SCHOOL_MASK_ARCANE] 	= {a=1,r=226/255,g=139/255,b=1};
	}
	self.records = {}
	self.vuln = Scorecard.vulnerableMobs
	self.EVENTS = {
		["SWING_DAMAGE"] = function(...)
			dmgDone,_,_,_,_,crit = select(1,...)
			return 6603, "Melee", 0x01, dmgDone, crit
		end,
		["SPELL_DAMAGE"] = function(...)
			spellID, spellName, spellSchool, dmgDone,_,_,_,_,crit  = select(1, ...)
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["RANGE_DAMAGE"] = function(...)
			spellID, spellName, spellSchool, dmgDone,_,_,_,_,crit  = select(1, ...)
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["SPELL_PERIODIC_DAMAGE"] = function(...)
			spellID, spellName, spellSchool, dmgDone,_,_,_,_,crit  = select(1, ...)
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["SPELL_HEAL"] = function(...)
			spellID, spellName, spellSchool, dmgDone, crit  = select(1, ...)
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["SPELL_PERIODIC_HEAL"] = function(...)
			spellID, spellName, spellSchool, dmgDone, crit  = select(1, ...)
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["SPELL_DRAIN"] = function(...)
			spellID, spellName, spellSchool, dmgDone  = select(1, ...)
			crit = nil
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["SPELL_LEECH"] = function(...)
			spellID, spellName, spellSchool, dmgDone  = select(1, ...)
			crit = nil
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["SPELL_PERIODIC_DRAIN"] = function(...)
			spellID, spellName, spellSchool, dmgDone  = select(1, ...)
			crit = nil
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["SPELL_PERIODIC_LEECH"] = function(...)
			spellID, spellName, spellSchool, dmgDone  = select(1, ...)
			crit = nil
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["DAMAGE_SHIELD"] = function(...)
			spellID, spellName, spellSchool, dmgDone,_,_,_,_,crit  = select(1, ...)
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
		["DAMAGE_SPLIT"] = function(...)
			spellID, spellName, spellSchool, dmgDone,_,_,_,_,crit  = select(1, ...)
			return spellID, spellName, spellSchool, dmgDone, crit
		end,
	}
	self:SetupFrames()
end

Scorecard:SetFuBarOption('iconPath', "Interface\\Icons\\INV_Scroll_02")
Scorecard:SetFuBarOption('defaultMinimapPosition', 260)
Scorecard:SetFuBarOption('tooltipHiddenWhenEmpty', true)
Scorecard:SetFuBarOption('hideWithoutStandby', true)
	
function Scorecard:Report()
	local name, rank, whatdb, db
	self:Print("Records:")
	whatdb = self.tabtext[PanelTemplates_GetSelectedTab(ScorecardFrame)]
	for id, t in pairs(Scorecard.db.char.records) do
		name, rank = GetSpellInfo(id)
		if t[whatdb] then
			self:Print(id, name, rank)
			db = t[whatdb]
			self:CustomPrint(self.colors[t.school].r, self.colors[t.school].g, self.colors[t.school].b, nil, nil, ", ", db.hit.damage, db.crit.damage)
		end
	end
end

function Scorecard:RefreshConfig()
	local sn, sr, si, complete
	--Populate delete table
	self.opts.args.delete.args = {}
	for k, v in pairs(Scorecard.db.char.records) do
		sn, sr, si = GetSpellInfo(k)
		self.opts.args.delete.args[k] = {
			type = 'execute',
			name = sn .. " " .. sr,
			desc = "ID: " .. k,
			buttonText = "Delete",
			func = function() 
				complete = self:DeleteRecord(k) 
				sn, sr = GetSpellInfo(k)
				if complete then
					self:Print("Records for " .. sn .. " " .. sr .. " deleted.")
					self:RefreshConfig()
					self:UpdateFuBarText()
					if self.frame:IsShown() then
						self:OnTabClick()
					end
				else
					self:Print("Error deleting " .. sn .. " " .. sr)
				end
			end
		}
	end
	--populate blacklist table
	self.opts.args.blacklist.args = {}
	for k, v in pairs(Scorecard.db.char.records) do
		sn, sr, si = GetSpellInfo(k)
		self.opts.args.blacklist.args[k] = {
			type = 'execute',
			name = sn .. " " .. sr,
			desc = "ID: " .. k,
			buttonText = "Delete and Blacklist",
			confirmText = "Are you sure you want to blacklist " .. sn .. " " .. sr .. "?",
			func = function() 
				complete = self:DeleteRecord(k) 
				sn, sr = GetSpellInfo(k)
				if complete then
					self:Print("Records for " .. sn .. " " .. sr .. " deleted.")
					Scorecard.db.char.options.blacklist[k] = true
					self:Print(sn .. " " .. sr .. "Blacklisted")
					self:RefreshConfig()
					self:UpdateFuBarText()
					if self.frame:IsShown() then
						self:OnTabClick()
					end
				else
					self:Print("Error deleting " .. sn .. " " .. sr)
				end
			end
		}
	end
	self.opts.args.blacklist.args.currentBlacklist = {
		name = "Current Blacklist",
		desc = "Ignored spells.",
		type = 'group',
		args = {},
	}
	--Populate current Blacklist table
	self.opts.args.blacklist.args.currentBlacklist.args = {}
	for k, v in pairs(Scorecard.db.char.options.blacklist) do
		sn, sr, si = GetSpellInfo(k)
		self.opts.args.blacklist.args.currentBlacklist.args[k] = {
			type = 'execute',
			name = sn .. " " .. sr,
			desc = "ID: " .. k,
			buttonText = "Remove from Blacklist",
			func = function()  
				sn, sr = GetSpellInfo(k)
				Scorecard.db.char.options.blacklist[k] = nil
				self:Print(sn .. " " .. sr .. " removed from Blacklist")
				self:RefreshConfig()
				self:UpdateFuBarText()
				if self.frame:IsShown() then
					self:OnTabClick()
				end
			end
		}
	end
	Scorecard:SetConfigTable(self.opts)
end

-- using LibRockConfig-1.0, this will automatically
-- open with the FuBar plugin when right-clicked.
self.opts = {
    name = "Scorecard",
    desc = "Tracks your record hits and crits.",
    type = 'group',
    args = {
		moveSplash = {
            type = 'execute',
            name = "Move the Splash Frame",
            desc = "Move the splash frame by dragging it around.",
			buttonText = "Click",
            func = function() Scorecard.splash.bg:Show() end
        },
        lowerRank = {
            type = 'boolean',
            name = "Show Lower Rank Spells",
            desc = "Show all ranks of spells available",
            get = function() return Scorecard.db.char.options.lowerRank end,
            set = function() 
				Scorecard.db.char.options.lowerRank = not Scorecard.db.char.options.lowerRank 
				self:UpdateFuBarText()
				self:OnTabClick()
			end
        },
		mySpells = {
            type = 'boolean',
            name = "Show My Spells",
            desc = "Show Spells from your Spellbook",
            get = function() return Scorecard.db.char.options.mySpells end,
            set = function() 
				Scorecard.db.char.options.mySpells = not Scorecard.db.char.options.mySpells 
				self:UpdateFuBarText()
				self:OnTabClick()
			end
        },
		notMySpells = {
            type = 'boolean',
            name = "Show Other Spells",
            desc = "Show Spells not in your Spellbook",
            get = function() return Scorecard.db.char.options.notMySpells end,
            set = function() 
				Scorecard.db.char.options.notMySpells = not Scorecard.db.char.options.notMySpells 
				self:UpdateFuBarText()
				self:OnTabClick()
			end
        },
		outputChoice = {
			name = "Output",
			desc = "Where to display new records",
			type = 'choice',
			get = function()
				return Scorecard.db.char.options.output
			end,
			set = function(value)
				Scorecard.db.char.options.output = value
			end,
			choices = {
				splash = "Splash Frame",
				errorframe = "UIErrorsFrame",
				text = "Chat Frame",
			},
			choiceOrder = {"splash", "errorframe", "text"},
		},
		input = {
			name = "Input Options",
			desc = "Choose what should be recorded",
			type = 'group',
			args = {
				playerInput = {
		            type = 'boolean',
		            name = "Record from Player",
		            desc = "Yourself",
					order = 1,
		            get = function() return Scorecard.db.char.options.input.player end,
		            set = function() 
						Scorecard.db.char.options.input.player = not Scorecard.db.char.options.input.player
					end
		        },
				petInput = {
		            type = 'boolean',
		            name = "Record from Pets",
		            desc = "Pets",
					order = 2,
		            get = function() return Scorecard.db.char.options.input.pet end,
		            set = function() 
						Scorecard.db.char.options.input.pet = not Scorecard.db.char.options.input.pet
					end
		        },
				guardianInput = {
		            type = 'boolean',
		            name = "Record from Guardians",
		            desc = "Totems and such",
					order = 3,
		            get = function() return Scorecard.db.char.options.input.guardian end,
		            set = function() 
						Scorecard.db.char.options.input.guardian = not Scorecard.db.char.options.input.guardian
					end
		        },
				objectInput = {
		            type = 'boolean',
		            name = "Record from Object",
		            desc = "Traps and such",
					order = 4,
		            get = function() return Scorecard.db.char.options.input.object end,
		            set = function() 
						Scorecard.db.char.options.input.object = not Scorecard.db.char.options.input.object
					end
		        },
			},
		},
		delete = {
			name = "Delete Spells",
		    desc = "Delete individual spells from your records.",
		    type = 'group',
		    args = {},
		},
		blacklist = {
			name = "Blacklist",
		    desc = "Ignore specific spells.",
		    type = 'group',
		    args = {},
		}
	}
}

function Scorecard:OnUpdateFuBarText()
	for k, v in pairs(self.maxHit) do
		v.spellID = 0
		v.dmg = 0
	end
	for k, v in pairs(self.maxCrit) do
		v.spellID = 0
		v.dmg = 0
	end
	for id, t1 in pairs(Scorecard.db.char.records) do
		if self:ShowSpell(id) then
			for control, t2 in pairs(t1) do
				if type(t2) == "table" then
					if t2.hit.damage > self.maxHit.All.dmg then
						self.maxHit.All.dmg = t2.hit.damage
						self.maxHit.All.spellID = id
					end
					if t2.crit.damage > self.maxCrit.All.dmg then
						self.maxCrit.All.dmg = t2.crit.damage
						self.maxCrit.All.spellID = id
					end
					if t2.hit.damage > self.maxHit[control].dmg then
						self.maxHit[control].dmg = t2.hit.damage
						self.maxHit[control].spellID = id
					end
					if t2.crit.damage > self.maxCrit[control].dmg then
						self.maxCrit[control].dmg = t2.crit.damage
						self.maxCrit[control].spellID = id
					end
				end
			end
		end
	end
	self:SetFuBarText("|cFFFFFFFF" .. self.maxHit.All.dmg .. "||" .. self.maxCrit.All.dmg)
end

function Scorecard:OnUpdateFuBarTooltip()
    GameTooltip:AddLine("Scorecard")
end

function Scorecard:OnFuBarClick(button)
	self:UpdateFuBarText()
	self:ToggleFrame()
end

function Scorecard:OnTabClick(i)
	if i then
		PanelTemplates_SetTab(self.frame, i)
	end
	local length = 0
	local name, rank, whatdb, db, num, icon
	record_db = {}
	whatdb = self.tabtext[PanelTemplates_GetSelectedTab(ScorecardFrame)]
	for id, t in pairs(Scorecard.db.char.records) do
		if self:ShowSpell(id) then
			if t[whatdb] then	
				name, rank, icon = GetSpellInfo(id)
				db = t[whatdb]
				length = length + 1
				record_db[length] = {
					["name"] = name,
					["rank"] = rank,
					["icon"] = icon,
					["school"] = t.school, 
					["hit"] = db.hit.damage, 
					["crit"] = db.crit.damage,
					["id"] = id
				}
			end
		end
	end
	table.sort(record_db, function(a, b)
		if a.name == b.name then
			return a.rank<b.rank
		end
		return a.name<b.name
	end)
	for k, v in pairs(self.frame.spells) do
		if v:GetID() > length then
			v:Hide()
		else
			v:Show()
			num = v:GetID() + self.SCROLLOFFSET
			if (record_db[num]) then
				v.icon:SetTexture(record_db[num].icon)
				
				v.name:SetTextColor(self.colors[record_db[num].school].r, self.colors[record_db[num].school].g, self.colors[record_db[num].school].b)
				v.name:SetText(record_db[num].name)
				if v.name:GetStringWidth() > 110 then
					repeat 
						v.name:SetText(string.sub(v.name:GetText(), 0, -2))
					until v.name:GetStringWidth() < (110 - self.frame.reference:GetStringWidth())
					v.name:SetText(v.name:GetText() .. "...")
				end
				
				v.rank:SetText(record_db[num].rank)
				--Coloring Bullllllllllllshit
				v.crit:SetTextColor(self.frame.title:GetTextColor())
				v.hit:SetTextColor(self.frame.title:GetTextColor())
				
				if self.maxHit[self.tabtext[PanelTemplates_GetSelectedTab(ScorecardFrame)]].spellID == record_db[num].id then
					v.hit:SetTextColor(1,1,1)
				end
				if self.maxHit.All.spellID == record_db[num].id and self.maxHit.All.dmg == record_db[num].hit then
					v.hit:SetTextColor(0,1,1)
				end
				if self.maxCrit[self.tabtext[PanelTemplates_GetSelectedTab(ScorecardFrame)]].spellID == record_db[num].id then
					v.crit:SetTextColor(1,1,1)
				end
				if self.maxCrit.All.spellID == record_db[num].id and self.maxCrit.All.dmg == record_db[num].crit then
					v.crit:SetTextColor(0,1,1)
				end
				v.hit:SetText(record_db[num].hit)
				v.crit:SetText(record_db[num].crit)
				
				if record_db[num].hit == 0 then
					v.hit:SetText("")
				end
				if record_db[num].crit == 0 then
					v.crit:SetText("")
				end
				v:SetScript("OnClick", function()
					self:OnSpellClick(record_db[v:GetID() + self.SCROLLOFFSET].id)
				end)
			else v:Hide() end
		end
	end
	if self.frame.info:IsShown() then 
		self:OnSpellClick(self.currentInfoSpell)
	end
end

function Scorecard:OnSpellClick(spellID)
	if spellID == self.currentInfoSpell then 
		self.currentInfoSpell = 0
		self.frame.info:Hide()
		return 
	end
	self.currentInfoSpell = spellID
	local t = self.db.char.records[spellID][self.tabtext[PanelTemplates_GetSelectedTab(ScorecardFrame)]]
	local tc = self.db.char.records[spellID].school
	local name, rank, icon = GetSpellInfo(spellID)
	self.frame.info.spellButton.label:	SetText(name .. " " .. rank)
	self.frame.info.spellButton.tex:	SetTexture(icon)
	self.frame.info.spellButton:SetScript("OnClick", function()
		SetItemRef("spell:"..spellID)
	end)
	self.frame.info.copyButton:SetScript("OnClick", function()
		self:InsertIntoChat(spellID)
	end)
	self.frame.info.spellButton.label: SetTextColor(self.colors[tc].r, self.colors[tc].g, self.colors[tc].b)
	if t.hit.damage > 0 then
		self.frame.info.hitTime:	SetText(date(self.timestampFormat, t.hit.timestamp))
		self.frame.info.hitTarget:	SetText(t.hit.name)
		self.frame.info.hitAmount:	SetText(t.hit.damage)
	else
		self.frame.info.hitTime:	SetText("")
		self.frame.info.hitTarget:	SetText("")
		self.frame.info.hitAmount:	SetText("")
	end
	if t.crit.damage > 0 then
		self.frame.info.critTime:	SetText(date(self.timestampFormat, t.crit.timestamp))
		self.frame.info.critTarget:	SetText(t.crit.name)
		self.frame.info.critAmount:	SetText(t.crit.damage)
	else
		self.frame.info.critTime:	SetText("")
		self.frame.info.critTarget:	SetText("")
		self.frame.info.critAmount:	SetText("")
	end
	self.frame.info:Show()
end

function Scorecard:ScrollUpdate()
	local length = 0
	local whatdb = self.tabtext[PanelTemplates_GetSelectedTab(ScorecardFrame)]
	for id, t in pairs(Scorecard.db.char.records) do
		if self:ShowSpell(id) then
			if t[whatdb] then	
				length = length + 1
			end
		end
	end
	FauxScrollFrame_Update(self.frame.scroll,length,22,16)
	self.SCROLLOFFSET = FauxScrollFrame_GetOffset(self.frame.scroll)
	if self.frame.info:IsShown() then 
		self:OnTabClick()
		self.frame.info:Show()	
	else
		self:OnTabClick()
	end
end

function Scorecard:ToggleFrame()
	if self.frame:IsShown() then
		self.frame:Hide()
	else
		self.frame:Show()
	end
end

function Scorecard:NewRecord(spellID, damage, controlType, crit, school)
	self:UpdateFuBarText()
	self:RefreshConfig()
	if self.frame:IsShown() then
		self:OnTabClick()
	end
	name, rank = GetSpellInfo(spellID)
	str = "%i\n|cffffffffNew %s %sRecord|r"
	textstr = "|cffffffffNew %s %sRecord:|r %i"
	critstr = ""
	if rank then
		name = name .. " " .. rank
	end
	if crit then 
		critstr = "Crit "
	end
	outputStr = string.format(str, damage, name, critstr)
	outputTextStr = string.format(textstr, name, critstr, damage)
	output = self.db.char.options.output
	if output == "splash" then
		self.splash:AddMessage(outputStr, self.colors[school].r,self.colors[school].g,self.colors[school].b,1,1)
	elseif output == "errorframe" then
		UIErrorsFrame:AddMessage(outputStr, self.colors[school].r,self.colors[school].g,self.colors[school].b,1,1)
	elseif output == "text" then
		self:CustomPrint(self.colors[school].r, self.colors[school].g, self.colors[school].b, nil, nil, ", ", outputTextStr)
	end
end

function Scorecard:DeleteRecord(spellID, controlType, crit)
	if crit == true then x = "crit" elseif crit == false then x = "hit" else nocrit = true end
	if controlType then
		if not nocrit then
			self.db.char.records[spellID][controlType][crit] = {
				["timestamp"] = 0.00,
				["name"] = "",
				["damage"] = 0,
			}
			return true
		else
			self.db.char.records[spellID][controlType] = {
				["hit"] = {
					["timestamp"] = 0.00,
					["name"] = "",
					["damage"] = 0,
				},
				["crit"] = {
					["timestamp"] = 0.00,
					["name"] = "",
					["damage"] = 0,
				}
			}
			return true
		end
	else
		self.db.char.records[spellID] = nil
		return true
	end
	return false
end

function Scorecard:ShowSpell(id)
	local rn, rr, ri
	local name, rank, icon = GetSpellInfo(id)
	rn, rr, ri = GetSpellInfo(name)
	if (not rn) and (self.db.char.options.notMySpells == false) then return false end
	if (rn == name) and (self.db.char.options.mySpells == false) then return false end
	if (self.db.char.options.lowerRank == false) then
		for k in pairs(self.db.char.records) do
			rn, rr, ri = GetSpellInfo(k)
			if (rn == name) and (ri == icon) and (rank < rr) then return false end --Not quite working yet, fix plz
		end
	end
	return true
end

function Scorecard:InsertIntoChat(spellID)
	local name, rank = GetSpellInfo(spellID)
	local t = self.db.char.records[spellID][self.tabtext[PanelTemplates_GetSelectedTab(ScorecardFrame)]]
	if not(rank == "") then
		rank = "(" .. rank .. ") "
	end
	local str = GetSpellLink(spellID) .. " " .. rank .. 
				self.tabtext[PanelTemplates_GetSelectedTab(ScorecardFrame)] .. " Records: "
	if t.hit then
		str = str .. t.hit.damage .. " hit on " .. t.hit.name --" on " .. date(self.timestampFormat, t.hit.timestamp)
		if t.crit.damage > 0 then str = str .. " || " end
	end
	if t.crit.damage > 0 then
		str = str .. t.crit.damage .. " crit on " .. t.crit.name --" on " .. date(self.timestampFormat, t.crit.timestamp)
	end
	--Taken from Historian, thanks Ixchael!
	if (ChatFrameEditBox == nil) then 
		return; 
	end;
	
	if not ChatFrameEditBox:IsVisible() then 
		ChatFrameEditBox:Show();
	end;
	
	ChatFrameEditBox:Insert(str);
end
